# -*- coding: utf-8 -*-
import logging
import os

# read environment
log_level = int(os.environ.get("YADISM_LOG_LEVEL", logging.INFO))
log_to_stdout = bool(os.environ.get("YADISM_LOG_STDOUT", True))
log_to_file = os.environ.get("YADISM_LOG_FILE")
silent_mode = bool(os.environ.get("YADISM_SILENT_MODE", False))

debug = bool(os.environ.get("DEBUG", False))

module_name = __name__.split(".")[0]
logger = logging.getLogger(module_name)
ekologger = logging.getLogger("eko")


def setup(log_level=log_level, log_to_stdout=log_to_stdout, log_to_file=log_to_file):
    """
    Init logging
    """
    logger.setLevel(log_level)
    ekologger.setLevel(log_level)
    # add rich logger
    if log_to_stdout:
        from rich.logging import RichHandler  # pylint: disable=import-outside-toplevel

        rh = RichHandler(log_level)
        rh.setFormatter(logging.Formatter("%(message)s", datefmt="[%X]"))
        logger.addHandler(rh)
        ekologger.addHandler(rh)
    # add file logger
    if log_to_file is not None:
        fh = logging.FileHandler(log_to_file)
        logger.addHandler(fh)
        ekologger.addHandler(fh)



# -*- coding: utf-8 -*-

import numba as nb

from ..intrinsic import fl_nc as intrinsic
from ..light import fl_nc as light
from ..partonic_channel import RSL, EmptyPartonicChannel
from . import partonic_channel as pc
from . import raw_nc


@nb.njit("f8(f8,f8[:])", cache=True)
def cg_NLO(z, _args):
    return raw_nc.clg1am0_a0(z)


@nb.njit("f8(f8,f8[:])", cache=True)
def cg_NNLO(z, args):
    L = args[0]
    return (raw_nc.clg2am0_aq(z) - raw_nc.clg2am0_af(z)) * L + raw_nc.clg2am0_a0(z)


class AsyGluonVV(pc.PartonicChannelAsy):
    def NLO(self):
        return RSL(cg_NLO, args=[self.L])

    def NNLO(self):
        return RSL(cg_NNLO, args=[self.L])


class AsyGluonAA(AsyGluonVV):
    pass


@nb.njit("f8(f8,f8[:])", cache=True)
def cps_NNLO(z, args):
    L = args[0]
    return (raw_nc.clps2am0_aq(z) - raw_nc.clps2am0_af(z)) * L + raw_nc.clps2am0_a0(z)


class AsySingletVV(pc.PartonicChannelAsy):
    def NNLO(self):
        return RSL(cps_NNLO, args=[self.L])


class AsySingletAA(AsySingletVV):
    pass


class PdfMatchingNonSinglet(EmptyPartonicChannel):
    pass


class LightNonSingletShifted(pc.PartonicChannelAsy):
    def NNLO(self):
        return light.NonSinglet(self.ESF, self.nf).NLO()


# @nb.njit("f8(f8,f8[:])", cache=True)
# def cns_NNLO(z, args):
#     L = args[0]
#     return raw_nc.clns2am0_aq(z) * L + raw_nc.clns2am0_a0(z)


# class AsyNonSingletMissing(pc.PartonicChannelAsy):
#     def NNLO(self):
#         return RSL(cns_NNLO, args=[self.L])


class MatchingIntrinsicSplus(pc.FMatchingQuark):
    ffns = intrinsic.Splus


class MatchingIntrinsicSminus(pc.FMatchingQuark):
    ffns = intrinsic.Sminus


class MatchingGluonSplus(pc.FMatchingGluon):
    ffns = intrinsic.Splus


class MatchingGluonSminus(pc.FMatchingGluon):
    ffns = intrinsic.Sminus



# -*- coding: utf-8 -*-
import LeProHQ
import numpy as np

from ..partonic_channel import RSL
from . import partonic_channel as pc


class NonSinglet(pc.NeutralCurrentBase):
    def NNLO(self):
        """
        |ref| implements NLO (heavy) non-singlet coefficient function, :cite:`felix-thesis`.
        """

        def dq(z, _args):
            if self.is_below_threshold(z):
                return 0.0
            return (
                self._FHprefactor
                / z
                * (4.0 * np.pi) ** 2
                * (LeProHQ.dq1("xF3", "VA", self._xi, self._eta(z)))
            )

        return RSL(dq)



# -*- coding: utf-8 -*-
from ..intrinsic import f3_nc as intrinsic
from ..light import f3_nc as light

# from ..partonic_channel import EmptyPartonicChannel
from . import partonic_channel as pc


class PdfMatchingNonSinglet(pc.PdfMatchingNonSinglet):
    pass


class LightNonSingletShifted(pc.PartonicChannelAsy):
    def NNLO(self):
        return light.NonSinglet(self.ESF, self.nf).NLO()


# class AsyNonSingletMissing(EmptyPartonicChannel):
#     pass
#     # def NNLO(self):
#     # TODO get F3 from g1


class MatchingIntrinsicRplus(pc.FMatchingQuark):
    ffns = intrinsic.Rplus


class MatchingIntrinsicRminus(pc.FMatchingQuark):
    ffns = intrinsic.Rminus



# -*- coding: utf-8 -*-
import banana.cfg
from banana import navigator as bnav

from . import navigator

banana_cfg = banana.cfg.cfg


def yelp(*args):
    """
    Help function (renamed to avoid clash of names) - short cut: h.
    """
    if len(args) == 0:
        print(
            f"""Welcome to yadmark navigator - the yadism benchmark skript!
Available variables:
    {bnav.help_vars}
    o = "{bnav.o}" -> query observables
Available functions:
    {bnav.help_fncs}
    dfl(id) - log as DataFrame
    simlogs(id) - find similar logs
    diff(id,id) - subtractig logs
    compare(id,id) - compare externals
    check_log(id) - check logs passed
    crashed_log(id) - print crashed logs
"""
        )
    elif len(args) == 1:
        return help(*args)
    return None


h = yelp

app = navigator.NavigatorApp(banana_cfg, "sandbox")

# register banana functions
bnav.register_globals(globals(), app)

# add my functions
check_log = app.check_log


def launch_navigator():
    """CLI Entry point"""
    return bnav.launch_navigator("yadism", "yadmark")
